/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.api.message;

import java.net.URI;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.w3c.dom.Document;

/**
 * The MessageExchange wrapper to handle properly MessageExchange against the
 * CDK.
 * 
 * @author rnaudin - EBM WebSourcing
 * @since 1.0
 */
public interface Exchange {

    static final String IN_MESSAGE_NAME = "in";

    static final String OUT_MESSAGE_NAME = "out";

    static final QName DEFAULT_FAULT_QNAME = new QName("http://petals.ow2.org/cdk/defaultFault",
            "0");

    static final String PROPERTY_PROVIDER_LINKTYPE = "org.ow2.petals.messaging.provider.linktype";

    /**
     * Put a Source object in the out message of the message exchange. If the
     * out normalized message is null, it is created here. The charset encoding
     * used to create the Source content of the message is automatically
     * extracted from the given xml message String. If no encoding specified in
     * the given xml String UTF-8 is used by default.
     * 
     * @param outContent
     *      the String content to put in the out message
     * @throws MessagingException
     *      if error occured during out message creation
     * 
     * @deprecated use the method ${@link #setOutMessageContent(Document))
     */
    void setOutMessageContent(final String outContent) throws MessagingException;

    /**
     * Put a Source object in the out message of the message exchange. If the
     * out normalized message is null, it is created here.
     * 
     * @param outContent
     *      the DOM document content to put in the out message
     * @throws MessagingException
     *      if error occured during out message creation
     */
    void setOutMessageContent(final Document outContent) throws MessagingException;

    /**
     * Put a Source object in the out message of the message exchange. If the
     * out normalized message is null, it is created here.
     * 
     * @param outContent
     *      the Source content to put in the out message
     * @throws MessagingException
     *      if error occured during out message creation
     */
    void setOutMessageContent(final Source outContent) throws MessagingException;

    /**
     * Put a Source object in the in message of the message exchange. If the in
     * normalized message is null, it is created here. The charset encoding used
     * to create the Source content of the message is automatically extracted
     * from the given xml message String. If no encoding specified in the given
     * xml String UTF-8 is used by default.
     * 
     * @param inContent
     *      the String content to put in the in message
     * @throws MessagingException
     *      if error occured during in message creation
     * 
     * @deprecated use the method ${@link #setInMessageContent(Document))
     */
    void setInMessageContent(final String inContent) throws MessagingException;

    /**
     * Put a Source object in the in message of the message exchange. If the in
     * normalized message is null, it is created here.
     * 
     * @param inContent
     *      the DOM Document content to put in the in message
     * @throws MessagingException
     *      if error occured during in message creation
     */
    void setInMessageContent(final Document inContent) throws MessagingException;

    /**
     * Put a Source object in the in message of the message exchange. If the in
     * normalized message is null, it is created here.
     * 
     * @param inContent
     *      the Source content to put in the in message
     * @throws MessagingException
     *      if error occured during in message creation
     */
    void setInMessageContent(final Source inContent) throws MessagingException;

    /**
     * Get the content of the "in" message as a String
     * 
     * @return the String representation of the "in" message content
     * @throws MessagingException
     *      if error occurred during conversion of the Source or if "in" message
     *      is null
     * @deprecated use the method ${@link #getInMessageContentAsDocument()
     */
    String getInMessageContentAsString() throws MessagingException;

    /**
     * Get the content of the "in" message as a DOM document. If the {@code
     * Source} is a {@code Stream}, no fork is processed, which make the {@code
     * Source} no more available.
     * 
     * @return the DOM document representation of the "in" message content
     * @throws MessagingException
     *      if error occurred during conversion of the Source or if "in" message
     *      is {@code null}
     */
    Document getInMessageContentAsDocument() throws MessagingException;

    /**
     * Get the content of the "in" message as a DOM document. If the {@code
     * Source} is a {@code Stream}, a fork is processed, which makes the {@code
     * Source} still available, but decreases the performance of this method as
     * the {@code Stream} is duplicated.
     * 
     * @return the DOM document representation of the "in" message content
     *@throws MessagingException
     *      if error occurred during conversion of the Source or if "in" message
     *      is {@code null}
     */
    Document getInMessageContentAsDocument(boolean forkSourceStream) throws MessagingException;

    /**
     * Get the content of the "in" message as a Source
     * 
     * @return the "in" message content
     * @throws MessagingException
     *      if "in" message is {@code null}
     */
    Source getInMessageContentAsSource() throws MessagingException;

    /**
     * Get the content of the "out" message as a String
     * 
     * @return the String representation of the "out" message content
     * @throws MessagingException
     *      if error occurred during String conversion of the Source or if "out"
     *      message is null
     * @deprecated use the method ${@link #getOutMessageContentAsDocument()
     */
    String getOutMessageContentAsString() throws MessagingException;

    /**
     * Get the content of the "out" message as a DOM document. If the {@code
     * Source} is a {@code Stream}, no fork is processed, which make the {@code
     * Source} no more available.
     * 
     * @return the DOM document representation of the "out" message content
     * @throws MessagingException
     *      if error occurred during conversion of the Source or if "in" message
     *      is {@code null}
     */
    Document getOutMessageContentAsDocument() throws MessagingException;

    /**
     * Get the content of the "out" message as a DOM document. If the {@code
     * Source} is a {@code Stream}, a fork is processed, which makes the {@code
     * Source} still available, but decreases the performance of this method as
     * the {@code Stream} is duplicated.
     * 
     * @return the DOM document representation of the "in" message content
     *@throws MessagingException
     *      if error occurred during conversion of the Source or if "in" message
     *      is {@code null}
     */
    Document getOutMessageContentAsDocument(boolean forkSourceStream) throws MessagingException;

    /**
     * Get the content of the "out" message as a Source
     * 
     * @return the "out" message content
     * @throws MessagingException
     *      if "out" message is null
     */
    Source getOutMessageContentAsSource() throws MessagingException;

    /**
     * Get all the input attachments in a {@link Set}
     * 
     * @param exchange
     * @return
     */
    Set<DataHandler> getInMessageAttachments() throws MessagingException;

    /**
     * Get the message exchange Pattern
     * 
     * @return the URI pattern. Can't be null
     * @throws MessagingException
     *      if no message exchange pattern is set
     */
    URI getExchangePattern() throws MessagingException;

    /**
     * Get the operation name (local part of the operation QName)
     * 
     * @return the operation name, NULL if no operation is set
     */
    String getOperationName();

    /**
     * Get the endpoint name specified in the message exchange
     * 
     * @return the endpoint name
     * @throws MessagingException
     *      if no Endpoint is set
     */
    String getEndpointName() throws MessagingException;

    /**
     * Extract incoming NormalizedMessage from given MessageExchange. Create the
     * message if it is null.
     * 
     * @return the incoming NormalizedMessage
     * 
     */
    NormalizedMessage getInMessage();

    /**
     * Set the input message
     * 
     * @param msg
     * @throws MessagingException
     */
    void setInMessage(final NormalizedMessage msg) throws MessagingException;

    /**
     * Extract outgoing NormalizedMessage from given MessageExchange. Create the
     * output message if message is null.
     * 
     * @return the outgoing NormalizedMessage
     * 
     */
    NormalizedMessage getOutMessage();

    /**
     * Set the output message
     * 
     * @param msg
     * @throws MessagingException
     */
    void setOutMessage(final NormalizedMessage msg) throws MessagingException;

    /**
     * Create a {@link Fault} with the given {@link Exception} and set it in the
     * {@link MessageExchange}
     * 
     * @param e
     *      the {@link Exception} to transform in {@link Fault}
     * @throws MessagingException
     */
    void setFault(Exception e) throws MessagingException;

    /**
     * Put a Map of attachements in the out message of the message exchange. If
     * the out normalized message is null, it is created here.
     * 
     * @param attachements
     *      the attachements to put in the out message
     * @throws MessagingException
     *      if error occured during out message creation
     */
    void setOutMessageAttachments(Map<String, DataHandler> attachements) throws MessagingException;

    /**
     * Put a Map of attachements in the out message of the message exchange. If
     * the out normalized message is null, it is created here.
     * 
     * @param attachements
     *      the attachements to put in the out message
     * @throws MessagingException
     *      if error occured during out message creation
     */
    void setInMessageAttachments(Map<String, DataHandler> attachements) throws MessagingException;

    /**
     * Set the message exchange status to DONE
     * 
     * @throws MessagingException
     */
    void setDoneStatus() throws MessagingException;

    /**
     * Test if the message status is DONE
     * 
     * @return
     */
    boolean isDoneStatus();

    /**
     * Set the message exchange status to Error
     * 
     * @throws MessagingException
     */
    void setErrorStatus() throws MessagingException;

    /**
     * Test if the message status is ERROR
     * 
     * @return
     */
    boolean isErrorStatus();

    /**
     * Set the message exchange status to ACTIVE
     * 
     * @throws MessagingException
     */
    void setActiveStatus() throws MessagingException;

    /**
     * Test if the message status is ACTIVE
     * 
     * @return
     */
    boolean isActiveStatus();

    /**
     * 
     * @return true is the message role is provider
     */
    boolean isProviderRole();

    /**
     * 
     * @return true is the message role is consumer
     */
    boolean isConsumerRole();

    /**
     * 
     * @return
     */
    boolean isInOnlyPattern();

    /**
     * 
     * @return
     */
    boolean isRobustInOnlyPattern();

    /**
     * 
     * @return
     */
    boolean isInOutPattern();

    /**
     * True if an In message is set on the MessageExchange false otherwise
     */
    boolean isInMessage();

    /**
     * True if an Out message is set on the MessageExchange false otherwise
     */
    boolean isOutMessage();

    /**
     * 
     * @return
     */
    boolean isInOptionalOutPattern();

    /**
     * 
     * @return
     * @throws MessagingException
     */
    Fault createFault() throws MessagingException;

    /**
     * 
     * @return
     */
    ServiceEndpoint getEndpoint();

    /**
     * 
     * @return
     */
    Exception getError();

    /**
     * 
     * @return
     */
    String getExchangeId();

    /**
     * 
     * @return
     */
    Fault getFault();

    /**
     * 
     * @return
     */
    QName getInterfaceName();

    /**
     * 
     * @return
     */
    QName getOperation();

    /**
     * 
     * @return
     */
    URI getPattern();

    /**
     * 
     * @param name
     * @return
     */
    Object getProperty(String name);

    /**
     * 
     * @return
     */
    Set<?> getPropertyNames();

    /**
     * 
     * @return
     */
    MessageExchange.Role getRole();

    /**
     * 
     * @return
     */
    QName getService();

    /**
     * 
     * @return
     */
    ExchangeStatus getStatus();

    /**
     * 
     * @return
     */
    boolean isTransacted();

    /**
     * 
     * @param endpoint
     */
    void setEndpoint(ServiceEndpoint endpoint);

    /**
     * Put the given exception as Error part of the message exchange. Allowed
     * exception types are : java.*, org.w3c.* and org.xml.*.
     */
    void setError(Exception error);

    /**
     * 
     * @param fault
     * @throws MessagingException
     */
    void setFault(Fault fault) throws MessagingException;

    /**
     * 
     * @param interfaceName
     */
    void setInterfaceName(QName interfaceName);

    /**
     * 
     * @param name
     */
    void setOperation(QName name);

    /**
     * 
     * @param name
     * @param obj
     */
    void setProperty(String name, Object obj);

    /**
     * 
     * @param service
     */
    void setService(QName service);

    /**
     * 
     * @param status
     * @throws MessagingException
     */
    void setStatus(ExchangeStatus status) throws MessagingException;

    /**
     * 
     * @param name
     * @param value
     * @throws MessagingException
     */
    void setOutMessageProperty(String name, Object value) throws MessagingException;

    /**
     * 
     * @param properties
     * @throws MessagingException
     */
    void setOutMessageProperties(Map<String, Object> properties) throws MessagingException;

    /**
     * 
     * @param name
     * @return
     * @throws MessagingException
     */
    Object getOutMessageProperty(String name) throws MessagingException;

    /**
     * 
     * @return
     * @throws MessagingException
     */
    Set<?> getOutMessagePropertyNames() throws MessagingException;

    /**
     * 
     * @param name
     * @param value
     * @throws MessagingException
     */
    void setInMessageProperty(String name, Object value) throws MessagingException;

    /**
     * 
     * @param properties
     * @throws MessagingException
     */
    void setInMessageProperties(Map<String, Object> properties) throws MessagingException;

    /**
     * 
     * @param name
     * @return
     * @throws MessagingException
     */
    Object getInMessageProperty(String name) throws MessagingException;

    /**
     * 
     * @return
     * @throws MessagingException
     */
    Set<?> getInMessagePropertyNames() throws MessagingException;

    /**
     * Set In message Attachment.
     * 
     * @param name
     * @param dataHandler
     * @throws MessagingException
     */
    void addInMessageAttachment(String name, DataHandler dataHandler) throws MessagingException;

    /**
     * Set Out message Attachment.
     * 
     * @param name
     * @param dataHandler
     * @throws MessagingException
     */
    void addOutMessageAttachment(String name, DataHandler dataHandler) throws MessagingException;

    /**
     * Set In security subject.
     * 
     * @param subject
     */
    void setInSubject(Subject subject);

    /**
     * Set Out security subject.
     * 
     * @param subject
     */
    void setOutSubject(Subject subject);
}
